@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 Edscott Wilson Garcia
+ * Copyright (C) 2002-4 Edscott Wilson Garcia
  * EMail: edscott@imp.mx
  *
  * Copyright (c) 2003 Benedikt Meurer <benedikt.meurer@unix-ag.uni-siegen.de>
@@ -42,13 +42,12 @@
 #include <sys/mount.h>
 #endif
 
+#include <stdio.h>
 #include <errno.h>
 #ifdef HAVE_FSTAB_H
 #include <fstab.h>
 #endif
-#ifdef HAVE_LIMITS_H
 #include <limits.h>
-#endif
 #ifdef HAVE_MEMORY_H
 #include <memory.h>
 #endif
@@ -58,19 +57,10 @@
 #ifdef HAVE_PATHS_H
 #include <paths.h>
 #endif
-#include <stdio.h>
-#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
-#endif
-#ifdef HAVE_SIGNAL_H
 #include <signal.h>
-#endif
-#ifdef HAVE_STRING_H
 #include <string.h>
-#endif
-#ifdef HAVE_UNISTD_H
 #include <unistd.h>
-#endif
 
 #ifdef HAVE_SYS_FS_TYPES_H
 #include <sys/fs_types.h>
@@ -96,53 +86,67 @@
 #include "constants.h"
 #include "types.h"
 
-#include "add_node_contents.h"
-#include "entry.h"
-#include "gdir.h"
-#include "misc.h"
-#include "refresh.h"
-#include "tubo.h"
+#include "basic_row.h"
+
 #include "xfstab.h"
+
+#include "entry.h"
+#include "misc.h"
+#include "password_dialog.h" 
+#include "refresh.h"
+#include "goto.h"
+#include "tubo.h"
 #include "icons.h"
 
+static gboolean is_smb;
 
-static gdir_t fstat_gdir;
-extern int stop;
+static int childFD;
+static xfdir_t fstat_xfdir;
 static pid_t Gpid;
 static GtkTreeView *fstab_treeview;
 
 static gint watch_stop(gpointer data)
 {
-    tree_details_t *tree_details = get_tree_details(fstab_treeview);
 
     if(!tree_details->tubo_object) return FALSE;
-    if(stop)
+    if(tree_details->stop)
     {
-	char m[32];
-	sprintf(m, "%d\n", Gpid);
-	print_diagnostics(fstab_treeview, "xf_WARNING_ICON", _("Subprocess aborted, pid="), m, NULL);
+	gchar *m=g_strdup_printf("%d", Gpid);
+	print_diagnostics("xfce/warning", strerror(ECANCELLED)," pid=", m, "\n",NULL);
+	g_free(m);
 	/*cancelled = TRUE;*/
 	TuboCancel(tree_details->tubo_object, NULL);
-	if(Gpid)
-	    kill(Gpid, SIGHUP);	/*hack! */
-	stop = FALSE;
+	/*usleep(250); if(Gpid) kill(Gpid, SIGHUP);*/
+	tree_details->stop = FALSE;
 	tree_details->tubo_object = NULL;
-	hide_stop(tree_details->window);
+	hide_stop();
 	Gpid = 0;
-	cursor_reset(fstab_treeview);
+	cursor_reset();
 	return FALSE;
     }
-    set_progress(fstab_treeview, -1, -1);
+    set_progress(-1, -1);
     return TRUE;
 }
 
+
+
 static int operate_stderr(int n, void *data)
 {
     char *line;
     if(n)
 	return TRUE;		/* this would mean binary data */
     line = (char *)data;
-    print_diagnostics(fstab_treeview, "xf_ERROR_ICON", line, NULL);
+    D(printf("stderr:%s",line);)
+    if (strncmp(line,"Password:",strlen("Password:"))==0){
+	const char *p;
+    	print_diagnostics("xfce/warning",_("Sudo password requested"),"\n", NULL);
+	p=get_password(tree_details->window,_("Sudo password requested"));
+	if (p && strlen(p)) write(childFD,p,strlen(p));
+	write(childFD,"\n",strlen("\n"));
+
+	return TRUE;
+    }
+    if (line[0] != '\n') print_diagnostics("xfce/error", line, NULL);
     return TRUE;
 }
 static int operate_stdout(int n, void *data)
@@ -151,7 +155,8 @@ static int operate_stdout(int n, void *data)
     if(n)
 	return TRUE;		/* this would mean binary data */
     line = (char *)data;
-    print_diagnostics(fstab_treeview, "xf_WARNING_ICON", line, NULL);
+    D(printf("stdout:%s",line);)
+    print_diagnostics(NULL, line, NULL);
     return TRUE;
 }
 
@@ -162,19 +167,17 @@ static void fork_finished_function(pid_t pid)
     tree_entry_t *en;
     int status;
     GtkTreeIter iter;
-    tree_details_t *tree_details = get_tree_details(fstab_treeview);
     sprintf(m, "%d\n", pid);
-    /*print_diagnostics(fstab_treeview, "xf_INFO_ICON", _("Subprocess done, pid="), m, NULL);*/
 
     tree_details->tubo_object = NULL;
-    hide_stop(tree_details->window);
-    cursor_reset(fstab_treeview);
+    hide_stop();
+    cursor_reset();
     /* another wait just in case the one in tubo misses 
      * (shouldn't happen, but it does)*/
     
     waitpid(pid, &status, WNOHANG);
-    en= get_selected_entry(fstab_treeview, &iter);
-    status=is_mounted(en->path);
+    en= get_selected_entry(&iter);
+    status=is_mounted((const gchar *)en->path);
     if (status < 0) TOGGLE_MOUNTED(en->type);
     else {
 	    if (status) SET_MOUNTED(en->type);
@@ -187,9 +190,10 @@ static void fork_finished_function(pid_t pid)
     	GtkTreeModel *treemodel = gtk_tree_view_get_model(fstab_treeview);
     	treepath = gtk_tree_model_get_path(treemodel, &iter);
     	gtk_tree_view_collapse_row(fstab_treeview, treepath);
-    	remove_folder(fstab_treeview, &iter);
+    	prune_row(treemodel, &iter, NULL,en);
+	insert_dummy_row(treemodel, &iter, NULL,en,NULL,NULL);
     	gtk_tree_path_free(treepath);
-	update_icon(fstab_treeview,&iter);
+	update_icon(treemodel,&iter);
     }
     
 #ifdef DEBUG  
@@ -198,13 +202,16 @@ static void fork_finished_function(pid_t pid)
 }
 
 /* NOTE: linux will fail if HAVE_STATFS is before HAVE_GETMNTENT section */
-int is_mounted(char *mnt_point){
+G_MODULE_EXPORT
+int is_mounted (const gchar *mnt_point){
 	
 #if defined(HAVE_GETMNTENT) && defined(HAVE_STRUCT_MNTENT_MNT_DIR)
 	struct mntent *m;
 	const gchar *mnttab;
 	FILE *mtab_fd;
 	FILE *tab_file;
+
+	
 #if defined(MNT_MNTTAB)
 	mnttab = MNT_MNTTAB;
 #else
@@ -269,134 +276,159 @@ int is_mounted(char *mnt_point){
 #endif
 }
 
+
+G_MODULE_EXPORT
 int
-fstab_mount(GtkTreeView *treeview, char *mnt_point, gboolean umnt)
+fstab_mount (GtkTreeView *treeview, const gchar *mnt_point, gboolean umnt,tree_entry_t *en)
 {
-    gchar *argument[6];
+    gchar *argument[12];
     gchar **ap;
     int i;
-    tree_details_t *tree_details = get_tree_details(treeview);
+    gchar *g=NULL;
+    static gchar *sudo_prompt=NULL;
+    gboolean use_sudo=FALSE;
+    gchar *umount="umount",*mount="mount";
+    const char *smb_user;
+    gchar *smb_options=NULL;
+    static gchar *smb_browse=NULL;
+    /*GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);*/
 
+    g_free(smb_browse);
+    smb_browse=NULL;
+    /* this will set the is_smb variable: */
+    if (!is_in_fstab(mnt_point)) {
+	if (!is_smb) {
+	    print_diagnostics("xfce/error",strerror(ENODEV),":\n","no smbfs support detected in kernel\n",NULL);
+	    return 0;
+	}
+	if (en && IS_XF_NETSHARE(en->subtype))
+	{	    
+	    smb_browse=randomTmpName(NULL);
+	    unlink(smb_browse);
+	    print_diagnostics(NULL,"$mkdir ",smb_browse,"\n",NULL);
+	    mkdir(smb_browse, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
+	}
+    }
+    
+    if (getenv("XFFM_USE_SUDO") && strlen(getenv("XFFM_USE_SUDO"))) use_sudo=TRUE;
+    /*if (smb_browse) use_sudo=FALSE;*/
+
+    if (!getuid()) use_sudo=FALSE;
     ap = argument;
 
-    if (getenv("XFFM_USE_SUDO") && strlen(getenv("XFFM_USE_SUDO"))){
-	      /* FIXME: do a check whether if the command is sudo allowed
-	       * without password, if so then: */
+    if (use_sudo) {
+       if ((g = g_find_program_in_path("sudo")) != NULL){
 	    *ap++ = "sudo";
+	    *ap++ = "-S";
+	    g_free(g);
+       } else {
+	   print_diagnostics("xfce/error",strerror(ENOENT),": sudo",NULL);
+       }
     }
-		    
-
-    i=is_mounted(mnt_point);
+		 
+    
+    if (smb_browse) i=0;
+    else i=is_mounted(mnt_point);
+    
     if (i >= 0){
-      if (i) *ap++ = "umount";
-      else *ap++ ="mount";
+      if (i) *ap++ = umount;
+      else *ap++ =mount;
     } else {
-      if (umnt)	*ap++ = "umount";
-      else	*ap++ = "mount";
+      if (umnt)	*ap++ = umount;
+      else	*ap++ = mount;
+    }
+    if (is_smb && i==0) {
+	if (smb_browse) {
+	    *ap++="-t";
+	    *ap++="smbfs";
+	}
+	smb_user=get_smbuserpass(tree_details->window,en);
+	if (!smb_user || !strlen(smb_user)){
+	  smb_options=g_strdup_printf("guest");
+	} else {
+	  smb_options=g_strdup_printf("username=%s",smb_user);
+	}
+     	*ap++ = "-o";
+	*ap++ = smb_options;
     }
 
+#if 0
 #if !defined(HAVE_GETVFSENT) && !defined(STAT_STATVFS) 
     /* XXX - This test is BAD */
     /* -v is different on Solaris */
     *ap++ = "-v";
 #endif
-
-    *ap++ = mnt_point;
+#endif
+    *ap++ = (gchar *)mnt_point;
+    if (smb_browse){
+	*ap++ = smb_browse;
+    }
     *ap++ = NULL;
 
-    print_status(treeview,"xf_INFO_ICON",_("Mount:"), mnt_point, NULL);
+    print_diagnostics(NULL,"$",argument[0],NULL);
+    for (i=1;argument[i];i++){
+	if (strstr(argument[i],"username=")) print_diagnostics(NULL," username=*****",NULL);
+	else 
+	    print_diagnostics(NULL," ",argument[i],NULL);
+    }
+    print_diagnostics(NULL,"\n",NULL);
     fstab_treeview = treeview;
     
-    chdir ("/");
-    show_stop(tree_details->window);
-
+    chdir(GETWD);
+    show_stop();
+	
+    /*printf("DBG: envars=%s\n",env_vars[i]);
+    printf("DBG: value=%s\n",value);*/
+    
+    /* put in a \n to the password prompt to flush the pipe: fluusshh */
+    if (sudo_prompt) g_free(sudo_prompt);
+    sudo_prompt = g_strconcat("SUDO_PROMPT=","Password:","\n",NULL);
+    putenv(sudo_prompt);
     tree_details->tubo_object = 
 		Tubo(fork_function, 
 				(void *)argument, 
 				fork_finished_function, 
-				FALSE, 
+				&childFD, 
 				operate_stdout, 
-				operate_stderr);
+				operate_stderr,0,TRUE);
     	
-    g_timeout_add_full(0,260, 
-			(GtkFunction) watch_stop, 
-			(gpointer) treeview,
-			NULL);
+    g_timeout_add(260,(GtkFunction) watch_stop, (gpointer) treeview);
     	
     Gpid = TuboPID(tree_details->tubo_object);
+    g_free(smb_options);
+    if (smb_browse) {
+	go_to(tree_details->treestuff+get_tree_id(treeview),smb_browse);
+    }
     return(1);
 }
 
-#if 0 /* NOTYET */
-/*
- * Check if file system is already mounted
- */
-static gboolean
-is_mounted(const gchar *mountp)
+
+/* NOTE: Cygwin does have FS mounts, but they work differently from
+   all other environments. For the moment, we will not support any
+   mounting functionality, although in the future we might add
+   cygwin specific support
+*/
+#if defined (__CYGWIN__)
+
+G_MODULE_EXPORT
+int
+open_fstab (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *treepath, 
+	   gpointer user_data)
 {
-#if defined(HAVE_STATFS) && defined(HAVE_STRUCT_STATFS_F_MNTONNAME)
-	/*
-	 * The BSD way...
-	 */
-	struct statfs statfsbuf;
-
-	if (statfs(mountp, &statfsbuf) == 0) {
-		if (strcmp(statfsbuf.f_mntonname, mountp) == 0)
-			return(TRUE);
-	}
-#elif defined(HAVE_GETMNTENT) && defined(HAVE_STRUCT_MNTTAB_MNT_MOUNTP)
-	/*
-	 * The Solaris way...
-	 */
-	struct mnttab mbuf;
-	FILE *fp;
-
-	if ((fp = fopen("/etc/mnttab", "r")) != NULL) {
-		for (;;) {
-			if (getmntent(fp, &mbuf) != 0)
-				break;
-
-			if (strcmp(mbuf.mnt_mountp, mountp) == 0) {
-				(void)fclose(fp);
-				return(TRUE);
-			}
-		}
-	}
-
-	(void)fclose(fp);
-#elif defined(HAVE_GETMNTENT) && defined(HAVE_STRUCT_MNTENT_MNT_DIR)
-	/*
-	 * The Linux way...
-	 */
-	struct mntent *mptr;
-	FILE *fp;
-
-	if ((fp = setmntent("/proc/mounts", "r")) == NULL)
-		if ((fp = setmntent("/etc/mtab", "r")) == NULL)
-			return(FALSE);
-
-	for (mptr = getmntent(fp); mptr != NULL; mptr = getmntent(fp)) {
-		if (strcmp(mptr->mnt_dir, mountp) == 0) {
-			(void)endmntent(fp);
-			return(TRUE);
-		}
-	}
-
-	(void)endmntent(fp);
-#endif
-
-	return(FALSE);
+	fstat_xfdir.gl=NULL;
+	return(1);
 }
 
-static void
-read_fstab(...)
+G_MODULE_EXPORT
+gboolean
+is_in_fstab (const gchar *path)
 {
+	is_smb=FALSE;
+    return FALSE;
 }
-#endif
 
 /* NOTE: FreeBSD will fail if is HAVE_GETVFSENT before HAVE_STATFS */
-
-#if defined(HAVE_GETMNTENT) && defined(HAVE_STRUCT_MNTENT_MNT_DIR)
+#elif defined(HAVE_GETMNTENT) && defined(HAVE_STRUCT_MNTENT_MNT_DIR)
 
 /*
  * HP-UX, Linux, SunOS 4.x
@@ -443,7 +475,7 @@ read_fstab(...)
 #endif
 
 int
-open_fstab(GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *treepath, 
+open_fstab (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *treepath, 
 		gpointer user_data)
 {
 	/*struct mntent *mnt_mounted;*/
@@ -454,6 +486,7 @@ open_fstab(GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *treepath,
 	FILE *fstab_fd;
 	FILE *mtab_fd;
 	int i;
+    GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
 
 #if defined(MNT_MNTTAB)
 	mnttab = MNT_MNTTAB;
@@ -474,7 +507,7 @@ open_fstab(GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *treepath,
 		return(0);
 	}
 
-	fstat_gdir.pathc = 0;
+	fstat_xfdir.pathc = 0;
 	
 	for (mnt_struct = getmntent(fstab_fd); mnt_struct != NULL;
 			mnt_struct = getmntent(fstab_fd)) {
@@ -489,18 +522,19 @@ open_fstab(GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *treepath,
 		if (!g_file_test(mnt_struct->mnt_dir, G_FILE_TEST_IS_DIR))
 			continue;
 
-		fstat_gdir.pathc++;
+		fstat_xfdir.pathc++;
         }
 
         (void)endmntent(fstab_fd);
 	
-	en = get_entry(treeview, iter);
+	gtk_tree_model_get(treemodel, iter, ENTRY_COLUMN, &en, -1);
 
-	fstat_gdir.gl = (dir_t *)malloc(fstat_gdir.pathc * sizeof(dir_t));
+	fstat_xfdir.gl = (dir_t *)malloc(fstat_xfdir.pathc * sizeof(dir_t));
 
         if ((fstab_fd = setmntent(MNT_CHECKLIST,"r")) == NULL) {
 		fprintf(stderr, "Unable to open %s\n", MNT_CHECKLIST);
-		g_free(fstat_gdir.gl);
+		g_free(fstat_xfdir.gl);
+		fstat_xfdir.gl=NULL;
 		return(0);
 	}
 
@@ -517,25 +551,25 @@ open_fstab(GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *treepath,
 		if (!g_file_test(mnt_struct->mnt_dir, G_FILE_TEST_IS_DIR))
 			continue;
 
-    		fstat_gdir.gl[i].en = stat_entry(mnt_struct->mnt_dir, en->type);
-   	 	fstat_gdir.gl[i].pathv = g_strdup(mnt_struct->mnt_dir);
+    		fstat_xfdir.gl[i].en = stat_entry(mnt_struct->mnt_dir, en->type);
+   	 	fstat_xfdir.gl[i].pathv = g_strdup(mnt_struct->mnt_dir);
 
-		SET_XF_FSTAB(fstat_gdir.gl[i].en->type);
+		SET_XF_FSTAB(fstat_xfdir.gl[i].en->type);
 
 		if (strcmp(MNTTYPE_NFS, mnt_struct->mnt_type) == 0)
-			SET_NFS_TYPE(fstat_gdir.gl[i].en->subtype);
+			SET_NFS_TYPE(fstat_xfdir.gl[i].en->subtype);
 		else if (strcmp(MNTTYPE_SMBFS, mnt_struct->mnt_type) == 0)
-			SET_NFS_TYPE(fstat_gdir.gl[i].en->subtype);
+			SET_SMB_TYPE(fstat_xfdir.gl[i].en->subtype);
 		else if (strcmp(MNTTYPE_PROCFS, mnt_struct->mnt_type) == 0)
-			SET_PROC_TYPE(fstat_gdir.gl[i].en->subtype);
+			SET_PROC_TYPE(fstat_xfdir.gl[i].en->subtype);
 		else if (strcmp(MNTTYPE_DEV, mnt_struct->mnt_type) == 0)
-			SET_PROC_TYPE(fstat_gdir.gl[i].en->subtype);
+			SET_PROC_TYPE(fstat_xfdir.gl[i].en->subtype);
 		else if (strcmp(MNTTYPE_SHM, mnt_struct->mnt_type) == 0)
-			SET_PROC_TYPE(fstat_gdir.gl[i].en->subtype);
+			SET_PROC_TYPE(fstat_xfdir.gl[i].en->subtype);
 		else if (strcmp(MNTTYPE_SWAPFS, mnt_struct->mnt_type) == 0)
-			SET_PROC_TYPE(fstat_gdir.gl[i].en->subtype);
+			SET_PROC_TYPE(fstat_xfdir.gl[i].en->subtype);
 		else if (strcmp(MNTTYPE_CDFS, mnt_struct->mnt_type) == 0)
-			SET_CDFS_TYPE(fstat_gdir.gl[i].en->subtype);
+			SET_CDFS_TYPE(fstat_xfdir.gl[i].en->subtype);
 
 		i++;
 
@@ -543,39 +577,53 @@ open_fstab(GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *treepath,
 
         (void)endmntent(fstab_fd);
 
-#if 0
-/* MOUNTED flag not needed in this case */	
-	for (i = 0; i < fstat_gdir.pathc; i++)
-	{
-		/*
-		 * Check if filesystem is already mounted
-		 */
-		if ((mtab_fd = setmntent(mnttab, "r")) == NULL) {
-			fprintf(stderr, "Unable to open %s\n", mnttab);
-			continue;
-		}
+	add_contents_row(treemodel, iter, &fstat_xfdir);
+	erase_dummy_row(treemodel, iter, NULL);
+	
 
-		for (mnt_mounted = getmntent(mtab_fd); mnt_mounted != NULL;
-				mnt_mounted = getmntent(mtab_fd)) {
-			if (!strcmp(fstat_gdir.gl[i].pathv,
-				    mnt_mounted->mnt_dir)) {
-				printf("%s: already mounted\n",
-						mnt_mounted->mnt_dir);
-		    		TOGGLE_MOUNTED(fstat_gdir.gl[i].en->type);
-			}
-		}
-
-		(void)endmntent(mtab_fd);
-	}
-#endif
-
-	add_node_contents(treeview, iter, &fstat_gdir);
-
-	gdirfree(&fstat_gdir);
+	xfdirfree(&fstat_xfdir);
 
         return(1);
 }
 
+gboolean
+is_in_fstab (const gchar *path)
+{
+        struct mntent *mnt_struct;
+	FILE *fstab_fd;
+	gboolean result=FALSE;
+
+	is_smb=FALSE;
+	if ((fstab_fd = setmntent(MNT_CHECKLIST,"r")) == NULL) {
+		fprintf(stderr, "Unable to open %s\n", MNT_CHECKLIST);
+		return(0);
+	}
+
+	
+	for (mnt_struct = getmntent(fstab_fd); mnt_struct != NULL;
+			mnt_struct = getmntent(fstab_fd)) {
+		if (strcmp(MNTTYPE_SWAP, mnt_struct->mnt_type) == 0)
+			continue;
+		if (strcmp(MNTTYPE_SMBFS, mnt_struct->mnt_type) == 0)
+			is_smb=TRUE;
+#if defined(MNTTYPE_IGNORE)
+		if (strcmp(MNTTYPE_IGNORE, mnt_struct->mnt_type) == 0)
+			continue;
+#endif
+
+		if (!g_file_test(mnt_struct->mnt_dir, G_FILE_TEST_IS_DIR))
+			continue;
+		if (strcmp(path,mnt_struct->mnt_dir)==0) {
+		    result=TRUE;
+		    break;
+		}
+        }
+
+        (void)endmntent(fstab_fd);
+
+        return(result);
+}
+
 /* needs STAT_STATVFS to avert FreeBSD from falling in here,
  * but that does not work with fbsd 5.0, so use HAVE_SYS_STATFS_H
  * instead... */
@@ -591,7 +639,7 @@ open_fstab(GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *treepath,
 #define MNTTYPE_SWAP	"swap"
 
 int
-open_fstab(GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *treepath, 
+open_fstab (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *treepath, 
 		gpointer user_data)
 {
 	struct vfstab vbuf;
@@ -601,6 +649,7 @@ open_fstab(GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *treepath,
 	FILE *mnttab_fp;
 	struct stat st;
 	int i;
+    GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
 
 	if ((vfstab_fp = fopen("/etc/vfstab", "r")) == NULL)
 		return(0);
@@ -625,15 +674,15 @@ open_fstab(GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *treepath,
 
 	(void)rewind(vfstab_fp);
 
-	if ((fstat_gdir.gl = (dir_t *)malloc(i * sizeof(dir_t))) == NULL) {
+	if ((fstat_xfdir.gl = (dir_t *)malloc(i * sizeof(dir_t))) == NULL) {
 		(void)fclose(mnttab_fp);
 		(void)fclose(vfstab_fp);
 		return(0);
 	}
 
-	fstat_gdir.pathc = i;
+	fstat_xfdir.pathc = i;
 
-	en = get_entry(treeview, iter);
+	gtk_tree_model_get(treemodel, iter, ENTRY_COLUMN, &en, -1);
 
 	for (i = 0;;) {
 		if (getvfsent(vfstab_fp, &vbuf) != 0)
@@ -645,17 +694,17 @@ open_fstab(GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *treepath,
 		if (!g_file_test(vbuf.vfs_mountp, G_FILE_TEST_IS_DIR))
 			continue;
 
-    		fstat_gdir.gl[i].en = stat_entry(vbuf.vfs_mountp, en->type);
-   	 	fstat_gdir.gl[i].pathv = g_strdup(vbuf.vfs_mountp);
+    		fstat_xfdir.gl[i].en = stat_entry(vbuf.vfs_mountp, en->type);
+   	 	fstat_xfdir.gl[i].pathv = g_strdup(vbuf.vfs_mountp);
 
-		SET_XF_FSTAB(fstat_gdir.gl[i].en->type);
+		SET_XF_FSTAB(fstat_xfdir.gl[i].en->type);
 	
 		if (strcmp(MNTTYPE_NFS, vbuf.vfs_fstype) == 0)
-			SET_NFS_TYPE(fstat_gdir.gl[i].en->subtype);
+			SET_NFS_TYPE(fstat_xfdir.gl[i].en->subtype);
 		else if (strcmp(MNTTYPE_HSFS, vbuf.vfs_fstype) == 0)
-			SET_CDFS_TYPE(fstat_gdir.gl[i].en->subtype);
+			SET_CDFS_TYPE(fstat_xfdir.gl[i].en->subtype);
 		else if (strcmp(MNTTYPE_CACHEFS, vbuf.vfs_fstype) == 0)
-			SET_PROC_TYPE(fstat_gdir.gl[i].en->subtype);
+			SET_PROC_TYPE(fstat_xfdir.gl[i].en->subtype);
 
 		i++;
 		/*
@@ -668,20 +717,51 @@ open_fstab(GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *treepath,
 				break;
 
 			if (strcmp(mbuf.mnt_mountp, vbuf.vfs_mountp) == 0) 
-		    		TOGGLE_MOUNTED(fstat_gdir.gl[i].en->type);
+		    		TOGGLE_MOUNTED(fstat_xfdir.gl[i].en->type);
 		}
 	}
 
         (void)fclose(mnttab_fp);
         (void)fclose(vfstab_fp);
 	
-	add_node_contents(treeview, iter, &fstat_gdir);
+	add_contents_row(treemodel, iter, &fstat_xfdir);
+	erase_dummy_row(treemodel, iter, NULL);
 
-	gdirfree(&fstat_gdir);
+	xfdirfree(&fstat_xfdir);
 
         return(1);
 }
 
+gboolean
+is_in_fstab (const gchar *path)
+{
+	gboolean result=FALSE;
+	struct vfstab vbuf;
+	FILE *vfstab_fp;
+	is_smb=FALSE;
+
+	if ((vfstab_fp = fopen("/etc/vfstab", "r")) == NULL)
+		return(0);
+
+	while (1) {
+		if (getvfsent(vfstab_fp, &vbuf) != 0)
+			break;
+		
+		if(strcmp(MNTTYPE_SWAP, vbuf.vfs_fstype) == 0)
+			continue;
+
+		if (!g_file_test(vbuf.vfs_mountp, G_FILE_TEST_IS_DIR))
+			continue;
+		if (strcmp(path,vbuf.vfs_mountp)==0) {
+		    result=TRUE;
+		    break;
+		}
+
+	}
+	(void)fclose(vfstab_fp);
+        return(result);
+}
+
 #elif defined(HAVE_GETFSENT)
 
 /*
@@ -735,17 +815,20 @@ open_fstab(GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *treepath,
 #define MNTTYPE_SWAP	"swap"
 
 int
-open_fstab(GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *treepath, 
+open_fstab (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *treepath, 
 	   gpointer user_data)
 {
+    GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
+#if 0
 #if defined(HAVE_STATFS) && defined(HAVE_STRUCT_STATFS_F_MNTONNAME)
 	struct statfs statfsbuf;
+#endif
 #endif
 	struct fstab *fs;
    	tree_entry_t *en;
 	int i;
 
-	en = get_entry(treeview, iter);
+	gtk_tree_model_get(treemodel, iter, ENTRY_COLUMN, &en, -1);
 
 	if (!setfsent())
 		return(0);
@@ -764,12 +847,12 @@ open_fstab(GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *treepath,
 	if (!setfsent())
 		return(0);
 	
-	if ((fstat_gdir.gl = (dir_t *)malloc(i * sizeof(dir_t))) == NULL) {
+	if ((fstat_xfdir.gl = (dir_t *)malloc(i * sizeof(dir_t))) == NULL) {
 		endfsent();
 		return(0);
 	}
 
-	fstat_gdir.pathc = i;
+	fstat_xfdir.pathc = i;
 
 	for (i = 0, fs = getfsent(); fs != NULL; 
 			fs = getfsent()) {
@@ -779,25 +862,25 @@ open_fstab(GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *treepath,
 		if (!g_file_test(fs->fs_file, G_FILE_TEST_IS_DIR))
 			continue;
 
-    		fstat_gdir.gl[i].en = stat_entry(fs->fs_file, en->type);
-   	 	fstat_gdir.gl[i].pathv = g_strdup(fs->fs_file);
+    		fstat_xfdir.gl[i].en = stat_entry(fs->fs_file, en->type);
+   	 	fstat_xfdir.gl[i].pathv = g_strdup(fs->fs_file);
 
-		SET_XF_FSTAB(fstat_gdir.gl[i].en->type);
+		SET_XF_FSTAB(fstat_xfdir.gl[i].en->type);
 
 		if (strcmp(MNTTYPE_CDFS, fs->fs_vfstype) == 0)
-			SET_CDFS_TYPE(fstat_gdir.gl[i].en->subtype);
+			SET_CDFS_TYPE(fstat_xfdir.gl[i].en->subtype);
 		else if (strcmp(MNTTYPE_CODAFS, fs->fs_vfstype) == 0)
-			SET_NFS_TYPE(fstat_gdir.gl[i].en->subtype);
+			SET_NFS_TYPE(fstat_xfdir.gl[i].en->subtype);
 		else if (strcmp(MNTTYPE_KERNFS, fs->fs_vfstype) == 0)
-			SET_PROC_TYPE(fstat_gdir.gl[i].en->subtype);
+			SET_PROC_TYPE(fstat_xfdir.gl[i].en->subtype);
 		else if (strcmp(MNTTYPE_MFS, fs->fs_vfstype) == 0)
-			SET_PROC_TYPE(fstat_gdir.gl[i].en->subtype);
+			SET_PROC_TYPE(fstat_xfdir.gl[i].en->subtype);
 		else if (strcmp(MNTTYPE_NFS, fs->fs_vfstype) == 0)
-			SET_NFS_TYPE(fstat_gdir.gl[i].en->subtype);
+			SET_NFS_TYPE(fstat_xfdir.gl[i].en->subtype);
 		else if (strcmp(MNTTYPE_PROCFS, fs->fs_vfstype) == 0)
-			SET_PROC_TYPE(fstat_gdir.gl[i].en->subtype);
+			SET_PROC_TYPE(fstat_xfdir.gl[i].en->subtype);
 		else if (strcmp(MNTTYPE_SMBFS, fs->fs_vfstype) == 0)
-			SET_NFS_TYPE(fstat_gdir.gl[i].en->subtype);
+			SET_SMB_TYPE(fstat_xfdir.gl[i].en->subtype);
 #if 0
 		/* no longer needed to set lag in this case */
 #if defined(HAVE_STATFS) && defined(HAVE_STRUCT_STATFS_F_MNTONNAME)
@@ -807,7 +890,7 @@ open_fstab(GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *treepath,
 		if (statfs(fs->fs_file, &statfsbuf) == 0) {
 			if (strcmp(fs->fs_file, statfsbuf.f_mntonname) == 0) {
 				printf("%s: already mounted\n", fs->fs_file);
-		    		TOGGLE_MOUNTED(fstat_gdir.gl[i].en->type);
+		    		TOGGLE_MOUNTED(fstat_xfdir.gl[i].en->type);
 			}
 		}
 #endif
@@ -818,35 +901,72 @@ open_fstab(GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *treepath,
 
 	endfsent();
 	
-	add_node_contents(treeview, iter, &fstat_gdir);
+	add_contents_row(treemodel, iter, &fstat_xfdir);
+	erase_dummy_row(treemodel, iter, NULL);
 
-	gdirfree(&fstat_gdir);
+	xfdirfree(&fstat_xfdir);
 		
         return(1);
 }
+
+gboolean
+is_in_fstab (const gchar *path)
+{
+	gboolean result=FALSE;
+	struct fstab *fs;
+
+	is_smb=FALSE;
+	if (!setfsent()) return(0);
+
+	for ( fs = getfsent(); fs != NULL; fs = getfsent()) {
+		if (strcmp(MNTTYPE_SWAP, fs->fs_vfstype) == 0)
+			continue;
+		if (strcmp(MNTTYPE_SMBFS, fs->fs_vfstype) == 0)
+			is_smb=TRUE;
+		if (!g_file_test(fs->fs_file, G_FILE_TEST_IS_DIR))
+			continue;
+		
+		if (strcmp(path,fs->fs_file)==0) {
+		    result=TRUE;
+		    break;
+		}
+	}
+	endfsent();		
+        return(result);
+}
+
+
 #else
 
 int
-open_fstab(GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *treepath, 
+open_fstab (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *treepath, 
 	   gpointer user_data)
 {
-	/* make GCC happy */
-	(void)&fstat_gdir;
-
+	fstat_xfdir.gl=NULL;
 	return(1);
 }
 
+gboolean
+is_in_fstab (const gchar *path)
+{
+	is_smb=FALSE;
+    return FALSE;
+}
+
 #endif
 
+G_MODULE_EXPORT
 void
-on_mount                          (GtkMenuItem     *menuitem,
-                                        gpointer         user_data){
+on_mount   (GtkMenuItem     *menuitem, gpointer         user_data){
     GtkTreeIter iter;
     tree_entry_t *en;
-    GtkTreeView *treeview = get_treeview((GtkWidget *) menuitem);
-    en = get_selected_entry(treeview, &iter);
-    if(!en || (!IS_FSTAB_TYPE(en->type) && !IS_ROOT_TYPE(en->type)))
+    gint tree_id = get_active_tree_id();
+    GtkTreeView *treeview = tree_details->treestuff[tree_id].treeview;
+
+    en = get_selected_entry(&iter);
+    /*if(!en || (!IS_FSTAB_TYPE(en->type) && !IS_ROOT_TYPE(en->type)))*/
+    if(!en || !en->path)
 	    return;
-    fstab_mount(treeview,en->path,(int)((long)user_data));
+    fstab_mount(treeview,(const gchar *)en->path,(int)((long)user_data),en);
 }
 
