@@ -61,9 +61,20 @@
 #include "xfstab.h"
 #include "toolbar.h"
 
-/*#define DEBUG*/
+/* if themes.mime.xml does not exist...*/
+#define FALLBACK 1
 
 
+#define PIXBUF_HASH
+#ifdef PIXBUF_HASH
+static GHashTable *pixbuf_hash=NULL;
+typedef struct pixbuf_hash_t {
+    GdkPixbuf *pixbuf;
+    int size;
+    struct pixbuf_hash_t *next;
+}pixbuf_hash_t;
+#endif
+
 static GtkStyle *style;
 static gchar *icon_theme=NULL;
 static const gchar *composite_type_id(const gchar *path,int type,int subtype,const gchar *id){
@@ -149,15 +160,100 @@ gboolean is_image (gchar *file){
     return result;
 }
 
+static
+const gchar *get_thumbnail_path(char *file, int size){
+    gchar *xdg_dir;
+    gchar *cache_dir,*filename;
+    static gchar *thumbnail_path=NULL;
+    GString *gs;
+    gchar key[11];
 
+    xdg_dir=xfce_resource_save_location (XFCE_RESOURCE_CACHE,"/",TRUE);
+    cache_dir = g_build_filename(xdg_dir,"xfce4","xffm","thumbnails",NULL);  
+    g_free(xdg_dir);
+    mkdir(cache_dir, 0770);
+    if (!g_file_test(cache_dir,G_FILE_TEST_IS_DIR)) {
+	g_free(cache_dir);
+	return NULL;
+    }
+
+    /* thumbnails are not subject to thumbnailization: */
+    filename=g_path_get_dirname(file);
+    if (strcmp(cache_dir,filename)==0){
+	DBG("thumbnails cannot be thumbnailed:%s",file);
+	g_free(cache_dir);
+	g_free(filename);
+	return NULL;
+    }
+    g_free(filename);
+    
+    
+    gs = g_string_new(file);
+    sprintf(key, "%10u", g_string_hash(gs));
+    g_string_free(gs, TRUE);
+
+    filename=g_strdup_printf("%s-%d.png",key,size);
+    thumbnail_path = g_build_filename(cache_dir,filename,NULL);
+    g_free(filename);
+    g_free(cache_dir);
+    
+    return (const gchar *)thumbnail_path;
+ }
+
+/* some of the code in this function contributed by Thomas Leonard: */
+static
+void save_thumbnail(const gchar *thumbnail_path, const gchar *original_path, GdkPixbuf *tgt){
+	int original_width, original_height;
+	char *swidth, *sheight, *ssize, *smtime;
+	struct stat st;
+	GError *error=NULL;
+	
+	if (!thumbnail_path || !original_path) return;
+	DBG("creating thumbnail for %s (%s)",original_path,thumbnail_path);
+	
+	original_width = gdk_pixbuf_get_width(tgt);
+	original_height = gdk_pixbuf_get_height(tgt);
+
+	if (stat(original_path, &st) != 0) return;
+
+	swidth = g_strdup_printf("%d", original_width);
+	sheight = g_strdup_printf("%d", original_height);
+	ssize = g_strdup_printf("%ld", (long int)st.st_size);
+	smtime = g_strdup_printf("%ld", st.st_mtime);
+	
+	gdk_pixbuf_save(tgt,
+			thumbnail_path,
+			"png",
+			&error,
+			"tEXt::Thumb::Image::Width", swidth,
+			"tEXt::Thumb::Image::Height", sheight,
+			"tEXt::Thumb::Size", ssize,
+			"tEXt::Thumb::MTime", smtime,
+			"tEXt::Software", "Xfce4",
+			NULL);
+	if (error) g_error_free (error);
+	g_free(swidth);
+	g_free(sheight);
+	g_free(ssize);
+	g_free(smtime);
+}
+
+#define BIG_ENOUGH (96*96*3)
 G_MODULE_EXPORT
 GdkPixbuf *create_preview (char *file, int size)
 {
     GdkPixbuf *src, *tgt;
     int max_w, w, h;
-    int pw, ph;
+    int area,pw, ph;
     double scaleW, scaleH, scale;
     GError *error = NULL;
+    const gchar *thumbnail_path=get_thumbnail_path(file,size);
+    
+    if (thumbnail_path && g_file_test(thumbnail_path,G_FILE_TEST_EXISTS)){
+	DBG("retrieving thumbnail for %s (%s)",file,thumbnail_path);
+	tgt = gdk_pixbuf_new_from_file(thumbnail_path, NULL);
+	if (tgt) return tgt;
+    }
 
     switch (size)
     {
@@ -174,48 +270,7 @@ GdkPixbuf *create_preview (char *file, int size)
 	    max_w = w = 200, h = 100;
 	    break;
     }
-#if 0
-#ifdef HAVE_LIBRSVG
-    if (strstr(file,".svg")) {
-	
-	src = rsvg_pixbuf_from_file ((const gchar *)file,&error);
-	if (error) {
-	    /*g_warning("%s: %s",file,error->message);*/
-	    g_error_free(error);
-	    error=NULL;
-	}
-	/******************/
-	/* FIXME: this block of code repeats itself a few lines below 
-	 * there should be a better way to get the w and h of a svg file...*/
-    ph = gdk_pixbuf_get_height(src);
-    pw = gdk_pixbuf_get_width(src);
-    if ((ph > h || pw > w) && pw > 0 && ph > 0){
-      scaleH = (double)h / ph;
-      scaleW = (double)w / pw;
-      if (scaleW*pw > max_w) scaleW=(double)max_w / pw;
-      scale = (scaleH < scaleW) ? scaleH : scaleW;
-      h = scale * ph;
-      w = scale * pw;
-    } else return (src);
-    if(w < 10 || h < 10)
-    {
-	if (src) g_object_unref(G_OBJECT(src));
-	return NULL;
-    }
-	/********************/
-	tgt = rsvg_pixbuf_from_file_at_size((const gchar *)file,w,h,&error);
-	if (src) g_object_unref(G_OBJECT(src));
 
-	
-	if (error){
-	    /*g_warning("%s: %s",file,error->message);*/
-	    g_error_free(error);
-	    error=NULL;
-	}
-	return tgt;
-    }
-#endif
-#endif
     src = gdk_pixbuf_new_from_file(file, &error);
     if(!src)
     {
@@ -229,6 +284,7 @@ GdkPixbuf *create_preview (char *file, int size)
 
     ph = gdk_pixbuf_get_height(src);
     pw = gdk_pixbuf_get_width(src);
+    area=ph*pw;
     if ((ph > h || pw > w) && pw > 0 && ph > 0){
       scaleH = (double)h / ph;
       scaleW = (double)w / pw;
@@ -236,7 +292,10 @@ GdkPixbuf *create_preview (char *file, int size)
       scale = (scaleH < scaleW) ? scaleH : scaleW;
       h = scale * ph;
       w = scale * pw;
-    } else return (src);
+    } else {
+	if (area > BIG_ENOUGH) save_thumbnail(thumbnail_path,file,src);
+	return (src);
+    }
     if(w < 10 || h < 10)
     {
 	if (src) g_object_unref(G_OBJECT(src));
@@ -247,6 +306,8 @@ GdkPixbuf *create_preview (char *file, int size)
 	return NULL;
     tgt = gdk_pixbuf_scale_simple(src, w, h, GDK_INTERP_BILINEAR);
     if (tgt) g_object_unref(G_OBJECT(src));
+
+    if (area > BIG_ENOUGH) save_thumbnail(thumbnail_path,file,tgt);
     return tgt;
 }
 G_MODULE_EXPORT
@@ -327,7 +388,8 @@ insert_pixbuf_tag (		const gchar *id,
     int x_tgt=0,y_tgt=0; /* default: NW */
     GdkPixbuf *tag,*otag;
     int width, height;
-    
+   
+    if (!(*tgt)) return;
     width=gdk_pixbuf_get_width(*tgt);
     height=gdk_pixbuf_get_height(*tgt);
 
@@ -339,6 +401,7 @@ insert_pixbuf_tag (		const gchar *id,
     if (!otag) return;
     tag = gdk_pixbuf_scale_simple(otag, width/scale,height/scale, GDK_INTERP_BILINEAR);	
     g_object_unref(G_OBJECT(otag));
+    if (!tag) return;
     
     if (strcmp(where,"NE")==0) 
 	x_tgt=(scale-1)*width/scale, y_tgt=0;
@@ -346,41 +409,177 @@ insert_pixbuf_tag (		const gchar *id,
        	x_tgt=0, y_tgt=(scale-1)*width/scale;
     else if (strcmp(where,"SE")==0) 
 	x_tgt=(scale-1)*width/scale, y_tgt=(scale-1)*width/scale;
+    else if (strcmp(where,"E")==0) 
+	x_tgt=(scale-1)*width/scale, y_tgt=(scale-1)*width/scale/2;
     gdk_pixbuf_copy_area (tag,0,0,width/scale,height/scale,*tgt,x_tgt,y_tgt);
     g_object_unref(G_OBJECT(tag));
 }
 
-
+#ifdef FALLBACK
+GHashTable *fallback_hash(void){
+    static GHashTable *hardfallback_hash=NULL;
+    if (hardfallback_hash) return hardfallback_hash;
+    else {
+ 	gchar **p;
+	const gchar *hard_fallback[]={
+	      "application", "d-application.png",
+	      "audio", "d-audio.png",
+	      "image", "d-image.png",
+	      "inode", "f-memory.png",
+	      "message", "d-message.png",
+	      "model", "d-model.png",
+	      "multipart", "d-multipart.png",
+	      "text", "d-file.png",
+	      "video", "d-video.png",
+	      "xfce/default" , "d-xfce.png",
+	      "xfce/b-book" , "b-book-closed.png", 
+	      "xfce/find_result" , "b-find.png", 
+	      "xfce/b-find" , "b-find.png", 
+	      "xfce/b-frequent" , "b-frequent.png", 
+	      "xfce/b-fstab" , "b-fstab.png", 
+	      "xfce/b-home" , "b-home.png", 
+	      "xfce/b-network" , "b-network.png", 
+	      "xfce/b-recent" , "b-recent.png", 
+	      "xfce/b-trash" , "b-trash.png", 
+	      "xfce/b-xffm" , "xffm_icon.xpm", 
+	      "xfce/broken" , "d-symlink-broken.png", 
+	      "xfce/c-gid" , "c-gid.png", 
+	      "xfce/c-mode" , "c-mode.png", 
+	      "xfce/c-name" , "c-name.png", 
+	      "xfce/c-size" , "c-size.png", 
+	      "xfce/c-time" , "c-time.png", 
+	      "xfce/c-time32" , "c-time.png", 
+	      "xfce/c-uid" , "c-uid.png", 
+	      "xfce/cdrom" , "f-cdrom.png", 
+	      "xfce/closed_folder" , "dir-closed.png", 
+	      "xfce/disk" , "f-disk.png", 
+	      "xfce/downarrow" , "downarrow.png", 
+	      "xfce/dvd" , "f-dvd.png", 
+	      "xfce/error" , "gtk-dialog-error", 
+	      "xfce/executable" , "d-application.png", 
+	      "xfce/floppy" , "f-floppy.png", 
+	      "xfce/go-back" , "go-back.png", 
+	      "xfce/go-down" , "downarrow.png", 
+	      "xfce/go-forward" , "go-forward.png", 
+	      "xfce/go-home" , "go-home.png", 
+	      "xfce/go-jump" , "go-jump.png", 
+	      "xfce/go-start" , "go-start.png", 
+	      "xfce/go-up" , "go-up.png", 
+	      "xfce/info" , "a-info.png", 
+	      "xfce/ipc" , "d-symlink.png", 
+	      "xfce/link" , "go-forward.png", 
+	      "xfce/netws" , "p-terminal.xpm", 
+	      "xfce/nfs" , "f-netdisk.png", 
+	      "xfce/no-access" , "d-no-access.png", 
+	      "xfce/open_folder" , "dir-open.png", 
+#ifdef USE_TOOLBAR_PANEL
+	      "xfce/p-edit" , "p-edit.png", 
+	      "xfce/p-file1" , "p-file1.png", 
+	      "xfce/p-file2" , "p-file2.png", 
+	      "xfce/p-games" , "p-games.png", 
+	      "xfce/p-man" , "d-help.png", 
+	      "xfce/p-multimedia" , "p-multimedia.png", 
+	      "xfce/p-network" , "p-network.png", 
+	      "xfce/p-newmail" , "p-newmail.png", 
+	      "xfce/p-nomail" , "p-nomail.png", 
+	      "xfce/p-oldmail" , "p-oldmail.png", 
+	      "xfce/p-paint" , "p-paint.png", 
+	      "xfce/p-print" , "p-print.png", 
+	      "xfce/p-schedule" , "p-schedule.png", 
+	      "xfce/p-sound" , "p-sound.png", 
+	      "xfce/p-terminal" , "p-terminal.png", 
+	      "xfce/p-unknown" , "p-unknown.png", 
+#endif
+	      "xfce/process" , "f-memory.png", 
+	      "xfce/question" , "gtk-dialog-question", 
+	      "xfce/share" , "d-symlink.png", 
+	      "xfce/share_open" , "d-symlink.png", 
+	      "xfce/t-exec" , "t-exec.png", 
+	      "xfce/t-find" , "t-find.png", 
+	      "xfce/t-new-directory" , "t-new-directory.png", 
+	      "xfce/t-new-file" , "t-new-file.png", 
+	      "xfce/t-preferences" , "t-preferences.png", 
+	      "xfce/t-quit" , "t-quit.png", 
+	      "xfce/t-reload" , "t-reload.png", 
+	      "xfce/t-terminal" , "t-terminal.png", 
+	      "xfce/t-xfce-setting-show" , "t-xfce-setting-show.png", 
+	      "xfce/t-xfdiff" , "t-xfdiff.png", 
+	      "xfce/t-xfmime-edit" , "t-xfmime-edit.png", 
+	      "xfce/t-zoom-in" , "t-zoom-in.png", 
+	      "xfce/t-zoom-out" , "t-zoom-out.png", 
+	      "xfce/t-zoom-view" , "t-zoom-view.png", 
+	      "xfce/unknown" , "d-xfce.png", 
+	      "xfce/warning" , "gtk-dialog-warning", 
+	      "xfce/waste_basket_empty" , "d-waste_basket_empty.png", 
+	      "xfce/waste_basket_full" , "d-waste_basket_full.png", 
+	      "xfce/xfdiff" , "xfdiff.png", 
+	      "xfce/xffm" , "xffm_icon.xpm", 
+	      "t-pastelink.png" , "t-pastelink.png", /* FIXME: add type */
+	      "downarrow.png" , "downarrow.png", /* FIXME: add type */
+	      NULL,NULL
+	};
+	hardfallback_hash = g_hash_table_new(g_str_hash, g_str_equal);
+	
+	for (p=(gchar **)hard_fallback; *p; p+=2){
+	    g_hash_table_insert(hardfallback_hash, *p,*(p+1));	
+	}
+	return hardfallback_hash;
+    }   
+}
+    
+#endif
 static GdkPixbuf *icon_tell_cut(int size, const gchar *id, gboolean cut)
 {
     GtkIconSet *iconset;
     int gtksize;
+    GdkPixbuf *pixbuf;
+    
     if (!id || !strlen(id)) return NULL;
-    iconset = MIME_ICON_get_iconset(id,tree_details->window);
-   
+#ifdef PIXBUF_HASH
+    if (pixbuf_hash && !cut){
+	pixbuf_hash_t *pixbuf_hash_p = g_hash_table_lookup(pixbuf_hash,id);
+	if (pixbuf_hash_p){
+	    TRACE("pixbuf found for %s",id);
+	    for (;pixbuf_hash_p; pixbuf_hash_p= pixbuf_hash_p->next){
+		if (pixbuf_hash_p->size==size) break;
+	    }
+	    if (pixbuf_hash_p){
+		pixbuf=pixbuf_hash_p->pixbuf;
+		g_object_ref(pixbuf);
+		return pixbuf;
+	    } else DBG("pixbuf size %d not found",size);
+	}
+	
+    }
+#endif
 
     if (strncmp(id,"gtk-",strlen("gtk-"))==0){
-	return load_stock_icon(id ,size);
+	pixbuf=load_stock_icon(id ,size);
+	goto pixbuf_done;
 	/*return gtk_widget_render_icon(tree_details->window, id ,size, NULL);*/
     } 	
+    
 
     if (strstr(id,"/composite")) {
 	gchar *basic_type=g_strdup(id);
 	
 	GdkPixbuf *folder,*ofolder;
-	    
+
 	*(strstr(basic_type,"/composite"))=0;
 	
 	ofolder = icon_tell(size,basic_type);
-	folder=gdk_pixbuf_copy(ofolder);
-	g_object_unref(G_OBJECT(ofolder));
+	g_free(basic_type);
+	if (ofolder) {
+	  folder=gdk_pixbuf_copy(ofolder);
+	  g_object_unref(G_OBJECT(ofolder));
+	} else folder=NULL;
 	
 	
 	if (strstr(id,"/shows_hidden")){
-	    insert_pixbuf_tag("gtk-add",&folder,size,4,"NE");
+	    insert_pixbuf_tag("gtk-add",&folder,size,4,"E");
 	}
 	else if (strstr(id,"/has_hidden")) {
-	    insert_pixbuf_tag("gtk-remove",&folder,size,4,"NE");
+	    insert_pixbuf_tag("gtk-remove",&folder,size,4,"E");
 	}
 	
 	if (strstr(id,"/no_write")){
@@ -395,29 +594,40 @@ static GdkPixbuf *icon_tell_cut(int size, const gchar *id, gboolean cut)
 	else if (strstr(id,"/fstab_unmounted")){
 	    insert_pixbuf_tag("gtk-no",&folder,size,5,"NW");
 	}
-	if (cut){
+	if (cut && folder){   
 	    iconset=gtk_icon_set_new_from_pixbuf(folder);
 	    g_object_unref(G_OBJECT(folder));
-	} else return folder;
+	} 
+	else {
+	    pixbuf=folder;
+	    goto pixbuf_done;
+	}
     }
     
-    if (!iconset) {
+    iconset = MIME_ICON_get_iconset(id,tree_details->window);
+#ifdef FALLBACK
+    if (!iconset){
 	GdkPixbuf *pb;
-	pb = MIME_ICON_create_pixbuf (id);
+	gchar *b_id=g_strdup(id);
+	if (!strstr(b_id,"xfce/") && strchr(b_id,'/')) *strchr(b_id,'/')=0;
+	pb = MIME_ICON_create_pixbuf(g_hash_table_lookup(fallback_hash(),b_id));
+	g_free(b_id);
 	if (pb) {
 	    iconset=gtk_icon_set_new_from_pixbuf(pb);
 	    g_object_unref(G_OBJECT(pb));
 	}
     }
+#endif
     
     if(!iconset) 
     {
-        iconset = MIME_ICON_get_iconset("xfce/default",tree_details->window);
+        iconset = MIME_ICON_get_iconset("d-file.png",tree_details->window);
 /*#ifdef DEBUG
         printf("icon for %s not found in $theme/mime.xml\n",id); 
         printf("DBG: looking for iconset %s...%s\n",id,(iconset)?"found":"not found"); 
 #endif*/
     }
+    
 
     if(!style) style = gtk_style_new();
     switch (size)
@@ -438,7 +648,39 @@ static GdkPixbuf *icon_tell_cut(int size, const gchar *id, gboolean cut)
 	    /*gtksize=GTK_ICON_SIZE_SMALL_TOOLBAR; break;*/
     }
     if(!iconset) return NULL;
-    return gtk_icon_set_render_icon(iconset, style, GTK_DIR_RIGHT, (cut) ? GTK_STATE_INSENSITIVE : GTK_STATE_NORMAL, gtksize, NULL, NULL);
+    pixbuf = gtk_icon_set_render_icon(iconset, style, GTK_DIR_RIGHT, (cut) ? GTK_STATE_INSENSITIVE : GTK_STATE_NORMAL, gtksize, NULL, NULL);
+pixbuf_done:
+#ifdef PIXBUF_HASH
+    if (!cut) {
+	pixbuf_hash_t *pixbuf_hash_p,*p;
+	g_object_ref(pixbuf);
+	if (!pixbuf_hash){
+	    pixbuf_hash = g_hash_table_new(g_str_hash, g_str_equal);
+	}
+	pixbuf_hash_p = g_hash_table_lookup(pixbuf_hash,id);
+	if (!pixbuf_hash_p) {
+	    DBG("new pixbuf for %s",id);
+	    pixbuf_hash_p=(pixbuf_hash_t *)malloc(sizeof(pixbuf_hash_t));
+	    pixbuf_hash_p->size=size;
+	    pixbuf_hash_p->pixbuf=pixbuf;
+	    pixbuf_hash_p->next=NULL;
+	    g_object_ref(pixbuf);	    
+	    g_hash_table_insert(pixbuf_hash, g_strdup(id), (gpointer) pixbuf_hash_p);
+	    return pixbuf;
+	}
+	for (p=pixbuf_hash_p; p && p->next; p=p->next);
+	
+	/*if (p->size==size) g_warning("this does not happen");*/
+	
+	pixbuf_hash_p = p->next = (pixbuf_hash_t *)malloc(sizeof(pixbuf_hash_t));
+	pixbuf_hash_p->size=size;
+	pixbuf_hash_p->pixbuf=pixbuf;
+	pixbuf_hash_p->next=NULL;
+	g_object_ref(pixbuf);	    
+	return pixbuf;    
+    }
+#endif
+    return pixbuf;
 }
 
 G_MODULE_EXPORT
@@ -804,32 +1046,42 @@ GtkWidget *icon_image (char *id)
 G_MODULE_EXPORT
 void recreate_icons (GtkTreeView *treeview)
 {
+    int theme_status;
     GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
     /* create new icons */
     if (icon_theme){
 	    g_free(icon_theme);
     }
     icon_theme=g_strdup(tree_details->theme);
-    
-    if (!MIME_ICON_load_theme()){
-	    g_warning("Your installation is broken");
-    }
-    /* reset icons */
-    gtk_widget_freeze_child_notify((GtkWidget *) (treeview));
-    gtk_tree_model_foreach(treemodel, set_size_icons, (gpointer) (tree_details));
-    gtk_widget_thaw_child_notify((GtkWidget *) (treeview));
+    theme_status=MIME_ICON_load_theme();
+    switch (theme_status) {
+	case 0:
+	    g_warning("Your mime-icons installation is somehow broken");
+	    break;
+	case -1:
+#ifdef DEBUG
+	    printf("DBG: theme has not changed\n");
+#endif
+	    break;
+	case 1:
+	    /* reset icons */
+	    gtk_widget_freeze_child_notify((GtkWidget *) (treeview));
+	    gtk_tree_model_foreach(treemodel, set_size_icons, (gpointer) (tree_details));
+	    gtk_widget_thaw_child_notify((GtkWidget *) (treeview));
+	    break;
+   }
 }
 
 G_MODULE_EXPORT
 GdkPixbuf *load_stock_icon (const gchar *id, int size){
-    gchar *s,*p=NULL;
+    /*gchar *s,*p=NULL;*/
     GdkPixbuf *pixbuf=NULL;
     if (strncmp(id,"gtk-",strlen("gtk-"))!=0){
-	g_warning("load_stock_icon called on non stock item %d",id);
+	g_warning("load_stock_icon called on non stock item %s",id);
 	return NULL;
     }
+#if 0
     s = g_strconcat("stock_",id+strlen("gtk-"),".png",NULL);
-    /*for (p=s;*p;p++) if (*p=='-') *p='_';*/
     p=MIME_ICON_find_pixmap_file ((const gchar *)s);
     if (p) {
       pixbuf = icon_tell(size,s);
@@ -838,7 +1090,7 @@ GdkPixbuf *load_stock_icon (const gchar *id, int size){
     D(else g_warning("cannot locate %s",s);)
     g_free(s);
     if (pixbuf) return pixbuf;
-
+#endif
     pixbuf=gtk_widget_render_icon(tree_details->window,id,size, NULL);
     return pixbuf;
 }
