@@ -43,6 +43,7 @@
 #include "types.h"
 
 #include "modules.h"
+#include "smb_module.h"
 
 #include "menu.h"
 #include "menu_callbacks.h"
@@ -385,8 +386,10 @@ create_menu_entry(		char *name,
   /* get the application icon would look nice */
   {
       GdkPixbuf *p=load_stock_icon("gtk-execute",GTK_ICON_SIZE_MENU);
-      image = gtk_image_new_from_pixbuf (p);
-      g_object_unref(p);
+      if (p) {
+	image = gtk_image_new_from_pixbuf (p);
+	g_object_unref(p);
+      } else image=NULL;
   }
   /*image = gtk_image_new_from_stock ("gtk-execute", GTK_ICON_SIZE_MENU);*/
   gtk_widget_show (image);
@@ -491,8 +494,18 @@ void add_preferences_items (void)
     add_sort_callbacks();
     menu = WIDGET("preferences3_menu");
     l = gtk_accel_groups_from_object((GObject *) tree_details->window);
-    if(!l || !l->data)	assert_not_reached();
-
+    if(!l || !l->data)	{
+	g_error("no accel group for window!");
+	assert_not_reached();
+    }
+    /* this button is still defined in glade: */
+    
+    gtk_widget_add_accelerator (lookup_widget(tree_details->window,"refresh3"), "activate", 
+			    (GtkAccelGroup *) (l->data),
+                              GDK_Z, GDK_CONTROL_MASK,
+                              GTK_ACCEL_VISIBLE);
+	    D(printf("DBG: adding CTL+%c\n",GDK_Z);)
+    
     for (i=0;preferences_titles[i].label;i++)
     {
 	w = gtk_check_menu_item_new_with_label(_(preferences_titles[i].label));
@@ -622,19 +635,58 @@ set_auto_command(			int j,
 	      
 
 	      if (queued && *queued && output_arg) command=g_strdup(app);
-	      else command=g_strdup(MIME_mk_command_line(app,path,FALSE,FALSE));
+	      else {
+		  if (strncmp(path,"//",strlen("//"))==0){
+		      gchar *netpath=g_build_filename(XF_SMBget_netfile_cache_dir(),strrchr(path,'/')+1,NULL);
+		      command=g_strdup(MIME_mk_command_line(app,netpath,FALSE,FALSE));
+		      g_free(netpath);
+		  }
+		  else command=g_strdup(MIME_mk_command_line(app,path,FALSE,FALSE));
+	      }
 	      
 	      /* check for duplicates */
-	      for (jj=0;jj<j;jj++){
-		gchar *set_command=g_object_get_data(G_OBJECT(WIDGET(auto_C_name[jj])), "command");
-#ifdef DEBUG
-		printf("j=%d: %s == %s\n",jj,set_command,command);
-#endif
-		if (strcmp(set_command,command)==0)
-		{ /* skip it */
-		  g_free(command);
-		  return j;
-		}	   
+	      {
+		gint argcp1;
+		gchar **argvp1;
+		GError *error=NULL;
+		 if (!g_shell_parse_argv ((const gchar *)command,&argcp1,&argvp1,&error)){
+		    g_free(command);
+		    g_error_free(error);
+		    g_strfreev(argvp1);
+		    return j;
+		 }
+	         for (jj=0;jj<j;jj++){
+		   gboolean is_duplicate=TRUE;
+		   gint ac,argcp2;
+		   gchar **argvp2;
+		   gchar *set_command=g_object_get_data(G_OBJECT(WIDGET(auto_C_name[jj])), "command");
+		   TRACE("j=%d: %s == %s\n",jj,set_command,command);
+		   if (!g_shell_parse_argv ((const gchar *)set_command,&argcp2,&argvp2,&error)){
+		    g_free(command);
+		    g_error_free(error);
+		    g_strfreev(argvp2);
+		    return j;
+		   }
+		   if (argcp2 != argcp1) {
+		    is_duplicate=FALSE;
+		   } else for (ac=0; ac<argcp2; ac++){
+		    if (strcmp(argvp2[ac],argvp1[ac])){
+			is_duplicate=FALSE;
+			break;
+		    }
+		   }
+		   TRACE("0x%x, 0x%x\n",argvp1,argvp2);
+		
+		   g_strfreev(argvp2);
+		   argvp2=NULL;
+		   /*if (strcmp(set_command,command)==0)*/
+		   if (is_duplicate) {
+		      g_free(command);
+		      return j;
+		   }
+		 } /* end for */
+		 g_strfreev(argvp1);
+		 argvp1=NULL;
 	      }
 	      
 	      if ((old_text=g_object_get_data(G_OBJECT(a),"command")) != NULL) g_free(old_text);
@@ -857,7 +909,6 @@ void do_select_popup (GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
     showit(popup, "columns1");
     showit(popup, "branches1");
 #endif
-    set_column_menu(tree_id);
     
 
     if (invalid_input || tree_details->selectionOK==0){
@@ -927,6 +978,7 @@ void do_select_popup (GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
         showit(popup, "show_hidden1");
         showit(popup, "paste1_menuitem"); 
 	showit(popup, "popup_separator1");
+	if (IS_NETFILE(en->subtype)) autostuff(popup,en);
 	if (IS_XF_NETSHARE(en->subtype)){
 	      showit(popup, "mountP");
 	}
@@ -1046,8 +1098,7 @@ G_MODULE_EXPORT
 void do_popup (GtkTreeView * treeview, GdkEventButton * event)
 {
     GtkTreePath *selectpath;
-    gtk_tree_view_get_path_at_pos(treeview, event->x, event->y, &selectpath, NULL, NULL, NULL);
-    do_select_popup(treeview, selectpath,event->time);
+    if (gtk_tree_view_get_path_at_pos(treeview, event->x, event->y, &selectpath, NULL, NULL, NULL)) do_select_popup(treeview, selectpath,event->time);
 }
 
 G_MODULE_EXPORT
