@@ -462,15 +462,24 @@ on_autotype_C                          (GtkMenuItem     *menuitem,
     gint argc;
     gchar **argv;
     GError *error=NULL;
+    tree_entry_t *tmp_en=NULL;
 
     en = get_selected_entry(&iter);
+    if (IS_NETFILE(en->subtype)){
+	const gchar *file=XF_SMBget_cache_file(treeview,en);
+	tmp_en = stat_entry ((gchar *)file, __LOCAL_TYPE);
+	en=tmp_en;
+    }
+    output_arg=g_object_get_data(G_OBJECT(menuitem),"output_arg");
     command=g_object_get_data(G_OBJECT(menuitem),"command");
     querypath=g_object_get_data(G_OBJECT(menuitem),"querypath");
     queued=g_object_get_data(G_OBJECT(menuitem),"queued");
-    output_arg=g_object_get_data(G_OBJECT(menuitem),"output_arg");
     output_ext=g_object_get_data(G_OBJECT(menuitem),"output_ext");
     if (output_arg && !output_ext) g_warning("output_arg && !output_ext not met"); 
-    if (querypath && !autofunction_workdir (treeview, en,querypath)) return;
+    if (querypath && !autofunction_workdir (treeview, en,querypath)) {
+	if (tmp_en) destroy_entry(tmp_en);
+	return;
+    }
     if (output_arg) {
 	gchar *outputdir=workdir;
 	workdir=g_path_get_dirname(en->path);
@@ -490,6 +499,7 @@ on_autotype_C                          (GtkMenuItem     *menuitem,
     if (!queued || *queued == FALSE){
        runvwd(workdir,argv);
 	g_strfreev (argv);
+	if (tmp_en) destroy_entry(tmp_en);
        return;
     } 
 
@@ -506,6 +516,7 @@ on_autotype_C                          (GtkMenuItem     *menuitem,
     chdir(GETWD);
     if (workdir && strcmp(workdir,GETWD)!=0) print_diagnostics(NULL,"$chdir ",GETWD,"\n",NULL);
     g_strfreev (argv);
+    if (tmp_en) destroy_entry(tmp_en);
 
 	
 }
