@@ -1,7 +1,7 @@
 /*  xffm4
  *
  *  Copyright (C) 2003 olivier fourdan <xfce@users.sourceforge.net>
- *  Copyright (C) 2003 edscott wilson garcia <edscott@users.sourceforge.net>
+ *  Copyright (C) 2003-2004 edscott wilson garcia <edscott@xfce.org>
  *  Copyright (C) 2002 Jasper Huijsmans <huysmans@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -23,10 +23,10 @@
 #include <config.h>
 #endif
 
-#ifdef XFCE_SUPPORT
 
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include <gtk/gtk.h>
@@ -37,18 +37,20 @@
 #include <libxfce4mcs/mcs-manager.h>
 #include <xfce-mcs-manager/manager-plugin.h>
 #include <libxfce4util/i18n.h>
+#include <libxfce4util/debug.h>
+#include <libxfce4util/utf8.h>
 #include <libxfce4util/util.h>
 
 #include "constants.h"
 #include "types.h"
 
+
 #ifndef XFFM_MCS_PLUGIN
 #define XFFM_MCS_PLUGIN
 #endif
 
+#define	OLDRCFILE	g_get_home_dir(),".xfce4","settings","xffm.xml"
 #define CHANNEL  "xffm"
-#define RCDIR    "settings"
-#define RCFILE   "xffm.xml"
 
 #ifndef PACKAGE_DATADIR
 #define PACKAGE_DATADIR "/usr/local/share/xffm4"
@@ -59,6 +61,13 @@
 
 static McsManager *mcs_manager;
 
+static GSList *remove_group=NULL;
+GtkWidget *radio_cancel,*radio_waste,*radio_unlink;
+GtkWidget *hide_book,*hide_local,*hide_network,
+    *hide_find,*hide_trash,*hide_fstab,
+    *hide_recent,*hide_frequent,
+    *hold_xterm,*diagnostics,*use_sudo;
+
 #if 0
 static GtkWidget **preference_w;
 static GtkWidget **view_w;
@@ -90,22 +99,19 @@ static GtkWidget *dialog = NULL;
 static McsManager *mcs_manager = NULL;
 
 enum {
-    TERM,
+    /* these are set by checkboxes */
+    XFFM_USE_SUDO,
+    XFFM_DEFAULT_UNLINK,
+    XFFM_HOLD_XTERM,
+    XFFM_DIAGNOSTICS_DISABLED,
+    /* these are editable: */
+    TERMCMD,
+    LANG,
     XFFM_HOME,
-    XFFM_STATUS_LINE_LENGTH,
-    XFFM_MAX_PREVIEW_SIZE,
     SMB_USER,
     SMB_CODESET,
-    XFFM_HOLD_XTERM,
-    XFFM_APPEND_FILES,
-    XFFM_HIDE_BOOK,
-    XFFM_HIDE_LOCAL,
-    XFFM_HIDE_NETWORK,
-    XFFM_HIDE_APPS,
-    XFFM_HIDE_FIND,
-    XFFM_HIDE_TRASH,
-    XFFM_HIDE_FSTAB,
-    XFFM_DEFAULT_UNLINK,
+    XFFM_STATUS_LINE_LENGTH,
+    XFFM_MAX_PREVIEW_SIZE,
     XFFM_OPTIONS
 };
 
@@ -117,22 +123,17 @@ typedef struct
 Options;
 
 static Options xffm_options[XFFM_OPTIONS]={
-	{ "TERM",                    NULL },
+	{ "XFFM_USE_SUDO",	     NULL },
+	{ "XFFM_DEFAULT_UNLINK",     NULL },
+	{ "XFFM_HOLD_XTERM",         NULL },
+	{ "XFFM_DIAGNOSTICS_DISABLED",                 NULL },
+	{ "TERMCMD",                 NULL },
+	{ "LANG",   NULL }, 
 	{ "XFFM_HOME",               NULL },
-	{ "XFFM_STATUS_LINE_LENGTH", NULL },
-	{ "XFFM_MAX_PREVIEW_SIZE",   NULL },
 	{ "SMB_USER",                NULL },
 	{ "SMB_CODESET",             NULL },
-	{ "XFFM_HOLD_XTERM",         NULL },
-	{ "XFFM_APPEND_FILES",       NULL },
-	{ "XFFM_HIDE_BOOK",          NULL },
-	{ "XFFM_HIDE_LOCAL",         NULL },
-	{ "XFFM_HIDE_NETWORK",       NULL },
-	{ "XFFM_HIDE_APPS",          NULL },
-	{ "XFFM_HIDE_FIND",          NULL },
-	{ "XFFM_HIDE_TRASH",         NULL },
-	{ "XFFM_HIDE_FSTAB",         NULL },
-	{ "XFFM_DEFAULT_UNLINK",     NULL }
+	{ "XFFM_STATUS_LINE_LENGTH", NULL }, /* FIXME: should be set by spinbutton */
+	{ "XFFM_MAX_PREVIEW_SIZE",   NULL }, /* FIXME: should be set by spinbutton */
 };
 
 /* Fwd. decl. */
@@ -142,17 +143,7 @@ static gboolean write_options(void);
 
 McsPluginInitResult mcs_plugin_init(McsPlugin *mp)
 {
-#if 0
-#ifdef ENABLE_NLS
-    bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
-#ifdef HAVE_BIND_TEXTDOMAIN_CODESET
-    bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-#endif
-    textdomain (GETTEXT_PACKAGE);
-#endif
-#else
     xfce_textdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
-#endif
 
     mcs_manager = mp->manager;
     create_channel();
@@ -165,16 +156,51 @@ McsPluginInitResult mcs_plugin_init(McsPlugin *mp)
     return MCS_PLUGIN_INIT_OK;
 }
 
+static void set_option_buttons(void){
+	int i;
+    for (i = 0; i < XFFM_OPTIONS; i++)
+    {
+	switch (i) {
+		case XFFM_DEFAULT_UNLINK:
+			if (strcmp(xffm_options[i].value,"waste")==0) gtk_toggle_button_set_active ((GtkToggleButton *)radio_waste,TRUE);
+			else if (strcmp(xffm_options[i].value,"unlink")==0) gtk_toggle_button_set_active ((GtkToggleButton *)radio_unlink,TRUE);
+			else gtk_toggle_button_set_active ((GtkToggleButton *)radio_cancel,TRUE);
+			break;
+		case XFFM_USE_SUDO:
+			if (strlen(xffm_options[i].value)) gtk_toggle_button_set_active ((GtkToggleButton *)use_sudo,TRUE);
+			break;
+		case XFFM_HOLD_XTERM:
+			if (strlen(xffm_options[i].value)) gtk_toggle_button_set_active ((GtkToggleButton *)hold_xterm,TRUE);
+			break;
+	}
+    }
+}
+
 static void create_channel(void)
 {
     McsSetting *setting;
 
-    const gchar *home = g_get_home_dir();
+    /*const gchar *home = g_get_home_dir();*/
+    gchar *xdg_dir=xfce_resource_save_location (XFCE_RESOURCE_CONFIG,"/",TRUE);
     gchar *rcfile;
     gint i;
 
-    rcfile = g_strconcat(home, G_DIR_SEPARATOR_S, ".xfce4", G_DIR_SEPARATOR_S, RCDIR, G_DIR_SEPARATOR_S, RCFILE, NULL);
-    mcs_manager_add_channel_from_file(mcs_manager, CHANNEL, rcfile);
+    rcfile=g_build_filename(xdg_dir,MCS_PLUGIN_FILE,NULL);
+    g_free(xdg_dir);
+    
+    if (!g_file_test (rcfile, G_FILE_TEST_EXISTS)){
+	g_free(rcfile);
+	rcfile=g_build_filename(OLDRCFILE,NULL);
+    }
+
+    if (g_file_test (rcfile, G_FILE_TEST_EXISTS))
+    {
+	mcs_manager_add_channel_from_file(mcs_manager, CHANNEL, rcfile);
+    }
+    else
+    {
+        mcs_manager_add_channel(mcs_manager, CHANNEL);
+    }
     g_free(rcfile);
 
     for (i = 0; i < XFFM_OPTIONS; i++)
@@ -185,27 +211,43 @@ static void create_channel(void)
             if(xffm_options[i].value)
             {
                 g_free(xffm_options[i].value);
+		xffm_options[i].value=NULL;
             }
-            xffm_options[i].value = g_strdup(setting->data.v_string);
+            if (getenv(xffm_options[i].name) && strlen(getenv(xffm_options[i].name)) && strcmp(getenv(xffm_options[i].name),setting->data.v_string)!=0)
+	    {
+		xffm_options[i].value = g_strdup(getenv(xffm_options[i].name));
+	    }
+	    else
+	    {
+		xffm_options[i].value = g_strdup(setting->data.v_string);
+	    }
         }
         else
-        {
-            xffm_options[i].value = g_strdup("");
+        {   /* if not previously defined, inherit from environment */
+            if(strcmp("LANG",xffm_options[i].name)==0 && getenv(xffm_options[i].name) ) 
+	    {
+		xffm_options[i].value = g_strdup(getenv(xffm_options[i].name));
+	    }
+	    else {
+		xffm_options[i].value = g_strdup("");
+	    }
             mcs_manager_set_string(mcs_manager, xffm_options[i].name, CHANNEL, xffm_options[i].value);
         }
+			
     }
 }
 
 static gboolean write_options(void)
 {
-    const gchar *home = g_get_home_dir();
+    /*const gchar *home = g_get_home_dir();*/
+    gchar *xdg_dir=xfce_resource_save_location (XFCE_RESOURCE_CONFIG,"/",TRUE);
     gchar *rcfile;
     gboolean result;
 
-    rcfile = g_strconcat(home, G_DIR_SEPARATOR_S, ".xfce4", G_DIR_SEPARATOR_S, RCDIR, G_DIR_SEPARATOR_S, RCFILE, NULL);
+    rcfile=g_build_filename(xdg_dir,MCS_PLUGIN_FILE,NULL);
+    g_free(xdg_dir);
     result = mcs_manager_save_channel_to_file(mcs_manager, CHANNEL, rcfile);
     g_free(rcfile);
-
     return result;
 }
 
@@ -246,8 +288,8 @@ static gboolean environment_changed(GtkCellRendererText *cell,
 
         gtk_tree_model_get (model, &iter, column, &old_text, -1);
 	g_free (old_text);
-
-	i = gtk_tree_path_get_indices (path)[0] + TERM;
+	old_text=NULL;
+	i = gtk_tree_path_get_indices (path)[0] + TERMCMD;
         if (xffm_options[i].value)
         {
             g_free(xffm_options[i].value);
@@ -276,14 +318,57 @@ static void dialog_delete(GtkWidget *dialog)
     dialog = NULL;
 }
 
+void remove_toggled(GtkToggleButton *togglebutton, gpointer user_data){
+	gchar *new_text = NULL;
+	if(gtk_toggle_button_get_active((GtkToggleButton *)radio_cancel)) {
+		new_text=g_strdup("");
+	} 
+	else if(gtk_toggle_button_get_active((GtkToggleButton *)radio_waste)) {
+		new_text=g_strdup("waste");
+	}
+	else if(gtk_toggle_button_get_active((GtkToggleButton *)radio_unlink)) {
+		new_text=g_strdup("unlink");
+	}
+        if (xffm_options[XFFM_DEFAULT_UNLINK].value)
+        {
+            g_free(xffm_options[XFFM_DEFAULT_UNLINK].value);
+        }
+	xffm_options[XFFM_DEFAULT_UNLINK].value = new_text;
+        mcs_manager_set_string(mcs_manager, xffm_options[XFFM_DEFAULT_UNLINK].name, CHANNEL, xffm_options[XFFM_DEFAULT_UNLINK].value);
+        mcs_manager_notify(mcs_manager, CHANNEL);
+        write_options();
+
+}
+
+void option_toggled(GtkToggleButton *togglebutton, gpointer user_data){
+	int i = (int)((long) user_data);
+	if (xffm_options[i].value)
+        {
+            g_free(xffm_options[i].value);
+        }
+	if (gtk_toggle_button_get_active(togglebutton)){
+	       	xffm_options[XFFM_DEFAULT_UNLINK].value = g_strdup("yes");
+	}
+	else {
+		xffm_options[XFFM_DEFAULT_UNLINK].value = g_strdup("");
+	}
+        mcs_manager_set_string(mcs_manager, xffm_options[i].name, CHANNEL, xffm_options[i].value);
+        mcs_manager_notify(mcs_manager, CHANNEL);
+        write_options();
+
+}
+
+
 static void run_xffm_settings_dialog(McsPlugin *mp)
 {
-    GtkWidget *header, *vbox,*frame, *sw;
+    GtkWidget *header, *vbox,*frame, *hbox, *sw;
     int i;
     GtkListStore *model;
     GtkTreeIter iter;
     GtkCellRenderer *renderer;
-    GtkWidget *treeview;
+    GtkWidget *treeview; 
+
+    xfce_textdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
 
     dialog = gtk_dialog_new_with_buttons(_("File manager"),
                                     	 NULL, GTK_DIALOG_NO_SEPARATOR, 
@@ -296,30 +381,128 @@ static void run_xffm_settings_dialog(McsPlugin *mp)
     
     /* pretty header */
     vbox = GTK_DIALOG(dialog)->vbox;
+#ifndef XFCE4_O_X_LIBRARIES
+    header = xfce_create_header(mp->icon, _("Xffm Settings"));
+#else
     header = create_header(mp->icon, _("Xffm Settings"));
+#endif
     gtk_box_pack_start(GTK_BOX(vbox), header, FALSE, TRUE, 0);
     
     add_spacer(GTK_BOX(vbox));
- 
+
+#if 0
     frame = xfce_framebox_new (_("Global Configuration"), FALSE);
     gtk_widget_show(frame);
     gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
 
+    vbox=gtk_vbox_new(FALSE,6);
+    label = (GtkWidget *) gtk_label_new(_("Default remove action:"));
+    gtk_widget_show(label);
+    gtk_box_pack_start ((GtkBox *)vbox,label,FALSE,FALSE,0);
+    
+    hbox=gtk_hbox_new(TRUE,2);
+    gtk_box_pack_start ((GtkBox *)vbox,hbox,FALSE,FALSE,0);
+#endif
+    frame = xfce_framebox_new (_("Default remove action:"), TRUE);
+    gtk_widget_show (frame);
+    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
+
+    hbox = gtk_hbox_new (FALSE, 6);
+    xfce_framebox_add (XFCE_FRAMEBOX (frame), hbox);
+    
+    radio_cancel = gtk_radio_button_new_with_label (NULL,_("Cancel"));
+    remove_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio_cancel));
+    gtk_box_pack_start ((GtkBox *)hbox,radio_cancel,FALSE,FALSE,0);
+    radio_waste = gtk_radio_button_new_with_label (remove_group,_("Wastebasket"));
+    remove_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio_waste));
+    gtk_box_pack_start ((GtkBox *)hbox,radio_waste,FALSE,FALSE,0);
+    radio_unlink = gtk_radio_button_new_with_label (remove_group,_("Unlink"));
+    remove_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio_unlink));
+    gtk_box_pack_start ((GtkBox *)hbox,radio_unlink,FALSE,FALSE,0);
+
+    gtk_widget_show_all (hbox);
+
+#if 0
+    label = (GtkWidget *) gtk_label_new(_("Other useful options:"));
+    gtk_widget_show(label);
+    gtk_box_pack_start ((GtkBox *)vbox,label,FALSE,FALSE,0);
+    
+    hbox=gtk_hbox_new(TRUE,2);
+    gtk_box_pack_start ((GtkBox *)vbox,hbox,FALSE,FALSE,0);
+#endif
+    add_spacer(GTK_BOX(vbox));
+
+    frame = xfce_framebox_new (_("Other useful options:"), TRUE);
+    gtk_widget_show (frame);
+    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
+
+    vbox = gtk_vbox_new (FALSE, 6);
+    gtk_widget_show (vbox);
+    xfce_framebox_add (XFCE_FRAMEBOX (frame), vbox);
+    
+    diagnostics = gtk_check_button_new_with_label (_("Disable diagnostics"));
+    hold_xterm = gtk_check_button_new_with_label (_("Hold xterms"));
+    use_sudo = gtk_check_button_new_with_label (_("Mount with sudo"));
+    gtk_box_pack_start ((GtkBox *)vbox,diagnostics,FALSE,FALSE,0);
+    gtk_box_pack_start ((GtkBox *)vbox,hold_xterm,FALSE,FALSE,0);
+    gtk_box_pack_start ((GtkBox *)vbox,use_sudo,FALSE,FALSE,0);
+
+    set_option_buttons();
+    
+    g_signal_connect(radio_cancel, "toggled", G_CALLBACK(remove_toggled), NULL);
+    g_signal_connect(radio_waste, "toggled", G_CALLBACK(remove_toggled), NULL);
+    g_signal_connect(radio_unlink, "toggled", G_CALLBACK(remove_toggled), NULL);
+
+
+    g_signal_connect(diagnostics, "toggled", G_CALLBACK(option_toggled), (gpointer)((long)(XFFM_DIAGNOSTICS_DISABLED)));
+    g_signal_connect(hold_xterm, "toggled", G_CALLBACK(option_toggled), (gpointer)((long)(XFFM_HOLD_XTERM)));
+    g_signal_connect(use_sudo, "toggled", G_CALLBACK(option_toggled), (gpointer)((long)(XFFM_USE_SUDO)));
+
+    gtk_widget_show_all(vbox);
+
+    {
+      gchar *g;
+      g=g_find_program_in_path("sudo");
+      if (g){
+	g_free(g);
+      } else {
+	  gtk_widget_hide(use_sudo);
+      }
+    }
+#if 0
+    xfce_framebox_add (XFCE_FRAMEBOX (frame), vbox);
+    label = (GtkWidget *) gtk_label_new(_("Environment variables:"));
+    gtk_widget_show(label);
+    gtk_box_pack_start ((GtkBox *)vbox,label,FALSE,FALSE,0);
+#endif
+    
+    vbox = GTK_DIALOG(dialog)->vbox;
+    
+    add_spacer(GTK_BOX(vbox));
+
+    frame = xfce_framebox_new (_("Environment variables:"), FALSE);
+    gtk_widget_show (frame);
+    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
+    
     sw = gtk_scrolled_window_new (NULL, NULL);
     gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
 					 GTK_SHADOW_ETCHED_IN);
     gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
-				    GTK_POLICY_AUTOMATIC,
-				    GTK_POLICY_AUTOMATIC);
-    gtk_container_set_border_width(GTK_CONTAINER(sw), BORDER);
+				    GTK_POLICY_NEVER,
+				    GTK_POLICY_NEVER);
     gtk_widget_show(sw);
     xfce_framebox_add (XFCE_FRAMEBOX (frame), sw);
 
+#if 0
+    gtk_container_set_border_width(GTK_CONTAINER(sw), BORDER);
+    gtk_box_pack_start ((GtkBox *)vbox,sw,FALSE,FALSE,0);
+    /*xfce_framebox_add (XFCE_FRAMEBOX (frame), sw);*/
+#endif
+    
     model = gtk_list_store_new (NUM_COLUMNS, G_TYPE_STRING, G_TYPE_STRING,
 			      G_TYPE_BOOLEAN);
-    for (i=0; i < XFFM_OPTIONS; i++) {
+    for (i=TERMCMD; i < XFFM_OPTIONS; i++) {
         gtk_list_store_append (model, &iter);
-
         gtk_list_store_set (model, &iter,
 			  COLUMN_VARIABLE,
 			  g_strdup (xffm_options[i].name),
@@ -356,7 +539,7 @@ static void run_xffm_settings_dialog(McsPlugin *mp)
 					         "editable", COLUMN_EDITABLE,
 					         NULL);
 
-    gtk_widget_set_size_request(treeview, 320, 200);
+    gtk_widget_set_size_request(treeview, 350, -1);
     gtk_widget_show(treeview);
     gtk_container_add (GTK_CONTAINER (sw), treeview);
 
@@ -364,4 +547,3 @@ static void run_xffm_settings_dialog(McsPlugin *mp)
     gtk_widget_show(dialog);
 }
 
-#endif
