@@ -21,6 +21,9 @@
 #include <string.h>
 #include <unistd.h>
 #include <dirent.h>
+#include <libxfcegui4/libxfcegui4.h>
+
+
 
 #include <gtk/gtk.h>
 #include <gmodule.h>
@@ -37,6 +40,7 @@
 #include "settings.h"
 #include "menu.h"
 #include "misc.h"
+#include "keybindings.h"
 #include "modules.h"
 #include "toolbar.h"
 #include "treeview.h"
@@ -52,6 +56,10 @@
 #endif
 #endif
 
+
+G_MODULE_EXPORT
+SessionClient *session_client;
+
 G_MODULE_EXPORT
 tree_details_t *tree_details=NULL;
 G_MODULE_EXPORT
@@ -77,11 +85,8 @@ static void finishit(int sig)
 	fprintf(stderr, "xffm: signal %d received. Cleaning up before exiting\n", sig);
 	cleanup_tmpfiles();
 	
-	if (sig == SIGSEGV) {
-		FILE *f=fopen(g_strconcat(g_get_home_dir(),LOG_FILE,NULL),"a");
-		fprintf(f, "signal %d received.\n", sig);
-		fclose(f);
-		assert_not_reached();
+	if (sig == SIGSEGV) {   /* drop a core file here */
+	    assert_not_reached();
 	}
 	exit(1);
     }
@@ -159,35 +164,6 @@ gboolean    on_size (GtkWidget *widget, GtkAllocation *allocation, gpointer user
     return TRUE;
 }
 
-G_MODULE_EXPORT
-void
-on_pane1_clicked                       (GtkButton       *button,
-                                        gpointer         user_data){
-    GtkWidget *hpaned=WIDGET("hpaned1");
-    tree_details->hpane=hpaned->allocation.width;
-    gtk_paned_set_position(GTK_PANED(hpaned), tree_details->hpane);
-    tree_details->hpane_ratio=1.0;
-}
-
-G_MODULE_EXPORT
-void
-on_pane0_clicked                       (GtkButton       *button,
-                                        gpointer         user_data){
-    GtkWidget *hpaned=WIDGET("hpaned1");
-    tree_details->hpane=0;
-    gtk_paned_set_position(GTK_PANED(hpaned), tree_details->hpane);
-    tree_details->hpane_ratio=0.0;
-}
-G_MODULE_EXPORT
-void
-on_pane10_clicked                       (GtkButton       *button,
-                                        gpointer         user_data){
-    GtkWidget *hpaned=WIDGET("hpaned1");
-    tree_details->hpane=hpaned->allocation.width/2;
-    gtk_paned_set_position(GTK_PANED(hpaned), tree_details->hpane);
-    tree_details->hpane_ratio=0.50;
-}
-
 static gboolean setup_dir(gchar *path){
 	struct stat st;
 	if(stat(path, &st) < 0)
@@ -205,7 +181,7 @@ static gboolean setup_dir(gchar *path){
 
 static void verify_directories(void){
     gchar *path;
-    gchar *xdg_dir=xfce_resource_save_location (XFCE_RESOURCE_CONFIG,"/",TRUE);
+    gchar *xdg_dir=xfce_resource_save_location (XFCE_RESOURCE_CONFIG,"/",FALSE);
     gchar *oldpath=g_build_filename(g_get_home_dir(),".xfce4","xffm",NULL);
 
     path=g_build_filename(xdg_dir,"xfce4",NULL);
@@ -291,16 +267,14 @@ int main(int argc, char *argv[])
 
     gtk_set_locale();
     gtk_init(&argc, &argv);
-
-    init_xffm_env();
- 
-#if 0
-    client_session = client_session_new(argc, argv, NULL /* data */ , SESSION_RESTART_IF_RUNNING, 60);
-    if(!session_init(client_session))
+    session_client = client_session_new (argc,argv, NULL,SESSION_RESTART_IF_RUNNING,1);
+    if(!session_init(session_client))
     {
 	g_message("Cannot connect to session manager");
     }
-#endif
+
+    init_xffm_env();
+ 
 
     {
 #ifdef HAVE_SIGACTION
@@ -320,7 +294,6 @@ int main(int argc, char *argv[])
 	sigaction(SIGSEGV,&act,NULL);
 	sigaction(SIGTERM,&act,NULL);
 	sigaction(SIGFPE,&act,NULL);
-	sigaction(SIGKILL,&act,NULL);
 	sigaction(SIGUSR1,&act,NULL);
 	sigaction(SIGUSR2,&act,NULL);
 #else
@@ -332,7 +305,6 @@ int main(int argc, char *argv[])
 	signal(SIGSEGV, finishit);
 	signal(SIGTERM, finishit);
 	signal(SIGFPE, finishit);
-	signal(SIGKILL, finishit);
 	signal(SIGUSR1, finishit);
 	signal(SIGUSR2, finishit);
 #endif
@@ -353,7 +325,7 @@ int main(int argc, char *argv[])
     tree_details->argv[0] = g_path_get_basename(argv[0]);
     if (strncmp(tree_details->argv[0],"lt-",strlen("lt-"))==0) 
 	  tree_details->argv[0] += strlen("lt-");
-    if (argc >=2) tree_details->argv[1] = argv[1];
+    if (argc >=2) tree_details->argv[1] = g_strdup(argv[1]);
     else tree_details->argv[1] = NULL;
     tree_details->preferences = DEFAULT_PREFERENCES;
     tree_details->theme = NULL;
@@ -371,7 +343,7 @@ int main(int argc, char *argv[])
     /*printf("DBG: mcs connected\n");   */
     
     if (!MIME_ICON_load_theme()){
-	g_warning("cannot load theme\n");
+	g_message("cannot load theme\n");
     }
 
     /* this is done by mime_icons module, but since that is not loaded yet...*/
@@ -395,9 +367,7 @@ int main(int argc, char *argv[])
       gchar *g=g_path_get_basename(argv[1]);
       if (g && strlen(g)){
         bookfile=g;
-#ifdef DEBUG
-        printf("DBG:using %s\n",bookfile);
-#endif
+        DBG("using %s\n",bookfile);
       } else {
 	g_free(g);
 	g=NULL;
@@ -429,9 +399,7 @@ int main(int argc, char *argv[])
 		 } else {
 		   g = g_strconcat(prg," ",argv[1],NULL);
 		 }
-#ifdef DEBUG
-		 printf("DBG:%s\n",g);
-#endif
+		 TRACE("%s\n",g);
 		 if (g_shell_parse_argv(g,&argcp,&argvp,&error)){
 		   execvp(argvp[0],argvp);
 		   g_strfreev(argvp);	   
@@ -462,9 +430,12 @@ int main(int argc, char *argv[])
       treestuff_t *treestuff=tree_details->treestuff+i;
       g_signal_connect ((gpointer) (treestuff->column[NAME_COLUMN]->button), "size-allocate", G_CALLBACK (on_name_size), (gpointer)((long)i));
     }
+
     if (tree_details->preferences & SHOW_TB1) toolbar_buttons();
 
+#ifdef USE_TOOLBAR_PANEL    
     if (tree_details->preferences & SHOW_TB2) parse_panel_config();
+#endif
 
     smart_menus();
     
@@ -521,7 +492,7 @@ int main(int argc, char *argv[])
     enable_pane_resize=TRUE;
 
     chdir(GETWD);
-    xffm_setenv("PWD",GETWD,FALSE);
+    xffm_setenv("PWD",(gchar *)GETWD,FALSE);
      set_widget_initial_state();
     /*turn_on_goto(treeview);*/
     turn_on_pasteboard();
@@ -540,10 +511,20 @@ int main(int argc, char *argv[])
     hideit(tree_details->window,"menubar2");
 #endif
   
-
-
+    {
+	GtkWidget *hpaned=WIDGET("hpaned1");
+	if (!strstr(tree_details->argv[0],"xffm")) tree_details->hpane=0;
+	DBG("pane size to %d\n",tree_details->hpane);
+	gtk_paned_set_position(GTK_PANED(hpaned), tree_details->hpane);
+	tree_details->hpane_ratio = (double)tree_details->hpane / (double)hpaned->allocation.width;
+    } 
     /* expand... */
-   if ((strstr(tree_details->argv[0],"xfsamba") ||
+    {
+      GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
+      GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
+      GtkTreePath *treepath=NULL;
+	
+      if ((strstr(tree_details->argv[0],"xfsamba") ||
 	 strstr(tree_details->argv[0],"xfbook") ||    
 	 strstr(tree_details->argv[0],"xftrash") ||    
 	 strstr(tree_details->argv[0],"xffrequent") ||    
@@ -552,14 +533,13 @@ int main(int argc, char *argv[])
 	 strstr(tree_details->argv[0],"xfglob") ||    
 	 strstr(tree_details->argv[0],"xffstab") )||
 	(strstr(tree_details->argv[0],"xffm") && argc >= 2)) 
-   {
-	GtkTreeIter iter;
-	tree_entry_t *en=NULL;
-	
+        {
+	  tree_entry_t *en=NULL;
+	  GtkTreeIter iter;
    
-	{
-	  if (strstr(tree_details->argv[0],"xffm") || strstr(tree_details->argv[0],"xftree")) 
+	  if (strstr(tree_details->argv[0],"xffm") || strstr(tree_details->argv[0],"xftree")){ 
 	      get_local_root(treeview, &iter, &en);
+	  }
 	  if (strstr(tree_details->argv[0],"xffrequent")) 
 	      get_frequent_root(treeview, &iter, &en);
 	  if (strstr(tree_details->argv[0],"xfrecent")) 
@@ -578,50 +558,50 @@ int main(int argc, char *argv[])
 	  if (strstr(tree_details->argv[0],"xffstab"))
 	      get_fstab_root(treeview, &iter, &en);
 #endif
-	  if (en){
-  	   GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
-           GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
-           GtkTreePath *treepath = gtk_tree_model_get_path(treemodel, &iter);
-    	   gtk_tree_view_expand_row(treeview, treepath, FALSE);  
-     	   gdk_flush();
-	   gtk_tree_view_scroll_to_cell(treeview, treepath, NULL, TRUE, 0.0, 0.0);
-    	   clear_dnd_selection_list();
-	   gtk_tree_selection_select_path (selection,treepath);
-           get_dnd_selection(treemodel, treepath, &iter, (gpointer) treeview);	   
-	   gtk_tree_view_set_cursor (treeview,treepath,NULL,FALSE);
- 	   gtk_tree_path_free(treepath); 
+	  if (!en) {
+	      treepath=gtk_tree_path_new_first();
+	      g_warning("Cannot find root element for branch");
 	  }
-	}
+	  else {
+	      treepath = gtk_tree_model_get_path(treemodel, &iter);
+
+	      gtk_tree_view_expand_row(treeview, treepath, FALSE);  
+     	      gdk_flush();
+	      gtk_tree_view_scroll_to_cell(treeview, treepath, NULL, TRUE, 0.0, 0.0);
+	  }
+      } 
+      else {
+	  treepath=gtk_tree_path_new_first();
+      }
+      clear_dnd_selection_list();
+      if (treepath) {
+	gtk_tree_selection_select_path (selection,treepath);
+	/*get_dnd_selection(treemodel, treepath, &iter, (gpointer) treeview);*/
+	gtk_tree_view_set_cursor (treeview,treepath,NULL,FALSE);
+	set_path_reference(treeview,treepath);
+	gtk_tree_path_free(treepath); 
+      }
    }
 
+
    for (i=0;i<TREECOUNT;i++){
      GtkTreeSortable *sortable=GTK_TREE_SORTABLE(tree_details->treestuff[i].treemodel);
      tree_details->treestuff[i].sort_column =GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID;
      gtk_tree_sortable_set_sort_column_id (sortable,tree_details->treestuff[i].sort_column,GTK_SORT_ASCENDING);
    }
-   /*
-   {
-       GtkWidget *label = gtk_bin_get_child(GTK_BIN(WIDGET("main_menu3")));
-       gchar *g=g_strdup_printf("%s (%s)",_("Main menu"),VERSION);
-       gtk_label_set_text((GtkLabel *)label,my_utf_string(g));
-       g_free(g);
-   }*/
 
-    {
-	GtkWidget *hpaned=WIDGET("hpaned1");
-	
-#ifdef DEBUG
-	printf("pane size to %d\n",tree_details->hpane);
-#endif
-        gtk_paned_set_position(GTK_PANED(hpaned), tree_details->hpane);
-        tree_details->hpane_ratio = (double)tree_details->hpane / (double)hpaned->allocation.width;
-    }
    turn_on();
    enable_write_config();
     
    write_local_xffm_config();
+
+   gtk_widget_grab_focus((GtkWidget *) treeview);
+      
+   tree_details->comm=0;
    gtk_main();
    mcs_stop_watch(); 
 
     return 0;
 }
+
+
